@@ -118,7 +118,7 @@ static JSBool
 boxed_new_resolve(JSContext *context,
                   JSObject  *obj,
                   jsid       id,
-                  uintN      flags,
+                  unsigned   flags,
                   JSObject **objp)
 {
     Boxed *priv;
@@ -327,7 +327,7 @@ static JSBool
 boxed_invoke_constructor(JSContext   *context,
                          JSObject    *obj,
                          const gchar *constructor_name,
-                         uintN        argc,
+                         unsigned     argc,
                          jsval       *argv,
                          jsval       *rval)
 {
@@ -347,7 +347,7 @@ static JSBool
 boxed_new(JSContext   *context,
           JSObject    *obj, /* "this" for constructor */
           Boxed       *priv,
-          uintN        argc,
+          unsigned     argc,
           jsval       *argv,
           jsval       *rval)
 {
@@ -896,7 +896,7 @@ define_boxed_class_fields (JSContext *context,
 
 static JSBool
 to_string_func(JSContext *context,
-               uintN      argc,
+               unsigned   argc,
                jsval     *vp)
 {
     JSObject *obj = JS_THIS_OBJECT(context, vp);
@@ -930,7 +930,6 @@ static struct JSClass gjs_boxed_class = {
     "GObject_Boxed",
     JSCLASS_HAS_PRIVATE |
     JSCLASS_NEW_RESOLVE |
-    JSCLASS_NEW_RESOLVE_GETS_START |
     JSCLASS_HAS_RESERVED_SLOTS(1),
     JS_PropertyStub,
     JS_PropertyStub,
