@@ -118,6 +118,16 @@ gjs_is_custom_property_quark (void)
     return val;
 }
 
+static GQuark
+gjs_object_priv_quark (void)
+{
+    static GQuark val = 0;
+    if (G_UNLIKELY (!val))
+        val = g_quark_from_static_string ("gjs::private");
+
+    return val;
+}
+
 /* Plain g_type_query fails and leaves @query uninitialized for
    dynamic types.
    See https://bugzilla.gnome.org/show_bug.cgi?id=687184 and
@@ -198,6 +208,13 @@ init_g_param_from_property(JSContext  *context,
     return VALUE_WAS_SET;
 }
 
+static inline ObjectInstance *
+proto_priv_from_js(JSContext *context,
+                   JSObject  *obj)
+{
+    return priv_from_js(context, JS_GetPrototype(context, obj));
+}
+
 /* a hook on getting a property; set value_p to override property's value.
  * Return value is JS_FALSE on OOM/exception.
  */
@@ -403,10 +420,8 @@ object_instance_new_resolve_no_info(JSContext       *context,
         if (base_info == NULL)
             continue;
 
-        if (g_base_info_get_type(base_info) != GI_INFO_TYPE_INTERFACE) {
-            g_base_info_unref(base_info);
-            continue;
-        }
+        /* An interface GType ought to have interface introspection info */
+        g_assert (g_base_info_get_type(base_info) == GI_INFO_TYPE_INTERFACE);
 
         iface_info = (GIInterfaceInfo*) base_info;
 
@@ -448,7 +463,7 @@ static JSBool
 object_instance_new_resolve(JSContext *context,
                             JSObject  *obj,
                             jsid       id,
-                            uintN      flags,
+                            unsigned   flags,
                             JSObject **objp)
 {
     GIFunctionInfo *method_info;
@@ -559,53 +574,17 @@ object_instance_new_resolve(JSContext *context,
      * https://bugzilla.gnome.org/show_bug.cgi?id=632922
      */
     if (method_info == NULL) {
-        GType *interfaces;
-        guint n_interfaces;
-        guint i;
-
-        interfaces = g_type_interfaces (priv->gtype, &n_interfaces);
-        for (i = 0; i < n_interfaces; i++) {
-            GIBaseInfo *base_info;
-            GIInterfaceInfo *iface_info;
-
-            base_info = g_irepository_find_by_gtype(g_irepository_get_default(),
-                                                    interfaces[i]);
-            if (!base_info)
-                continue;
-
-            if (g_base_info_get_type(base_info) != GI_INFO_TYPE_INTERFACE) {
-                g_base_info_unref(base_info);
-                continue;
-            }
-
-            iface_info = (GIInterfaceInfo*) base_info;
-
-            method_info = g_interface_info_find_method(iface_info, name);
-
-            g_base_info_unref(base_info);
-
-            if (method_info != NULL) {
-                gjs_debug(GJS_DEBUG_GOBJECT,
-                          "Found method %s in native interface %s",
-                          name, g_type_name(interfaces[i]));
-                break;
-            }
-        }
-        g_free(interfaces);
-    }
-
-    if (method_info != NULL) {
-        const char *method_name;
-
+        ret = object_instance_new_resolve_no_info(context, obj, objp,
+                                                  priv, name);
+        goto out;
+    } else {
 #if GJS_VERBOSE_ENABLE_GI_USAGE
         _gjs_log_info_usage((GIBaseInfo*) method_info);
 #endif
 
-        method_name = g_base_info_get_name( (GIBaseInfo*) method_info);
-
         gjs_debug(GJS_DEBUG_GOBJECT,
                   "Defining method %s in prototype for %s (%s.%s)",
-                  method_name,
+                  g_base_info_get_name( (GIBaseInfo*) method_info),
                   g_type_name(priv->gtype),
                   g_base_info_get_namespace( (GIBaseInfo*) priv->info),
                   g_base_info_get_name( (GIBaseInfo*) priv->info));
@@ -644,7 +623,7 @@ free_g_params(GParameter *params,
 static JSBool
 object_instance_props_to_g_parameters(JSContext   *context,
                                       JSObject    *obj,
-                                      uintN        argc,
+                                      unsigned     argc,
                                       jsval       *argv,
                                       GType        gtype,
                                       GParameter **gparams_p,
@@ -804,7 +783,7 @@ wrapped_gobj_toggle_notify(gpointer      data,
      * Or if !is_last_ref, we do not want to convert to a strong
      * ref since we want everything collected on runtime destroy.
      */
-    context = gjs_runtime_get_current_context(runtime);
+    context = gjs_runtime_get_context(runtime);
     if (!context)
         return;
 
@@ -851,7 +830,6 @@ init_object_private (JSContext *context,
 {
     ObjectInstance *proto_priv;
     ObjectInstance *priv;
-    JSObject *proto;
 
     JS_BeginRequest(context);
 
@@ -865,23 +843,14 @@ init_object_private (JSContext *context,
     gjs_debug_lifecycle(GJS_DEBUG_GOBJECT,
                         "obj instance constructor, obj %p priv %p", object, priv);
 
-    proto = JS_GetPrototype(context, object);
-    gjs_debug_lifecycle(GJS_DEBUG_GOBJECT, "obj instance __proto__ is %p", proto);
-
-    proto_priv = priv_from_js(context, proto);
-    if (proto_priv == NULL) {
-        gjs_debug(GJS_DEBUG_GOBJECT,
-                  "Bad prototype set on object? Must match JSClass of object. JS error should have been reported.");
-        priv = NULL;
-        goto out;
-    }
+    proto_priv = proto_priv_from_js(context, object);
+    g_assert(proto_priv != NULL);
 
     priv->gtype = proto_priv->gtype;
     priv->info = proto_priv->info;
     if (priv->info)
         g_base_info_ref( (GIBaseInfo*) priv->info);
 
- out:
     JS_EndRequest(context);
     return priv;
 }
@@ -929,7 +898,7 @@ associate_js_gobject (JSContext      *context,
 static JSBool
 object_instance_init (JSContext *context,
                       JSObject **object,
-                      uintN      argc,
+                      unsigned   argc,
                       jsval     *argv)
 {
     ObjectInstance *priv;
@@ -943,12 +912,7 @@ object_instance_init (JSContext *context,
     priv = init_object_private(context, *object);
 
     gtype = priv->gtype;
-    if (gtype == G_TYPE_NONE) {
-        gjs_throw(context,
-                  "No GType for object '%s'???",
-                  g_base_info_get_name( (GIBaseInfo*) priv->info));
-        return JS_FALSE;
-    }
+    g_assert(gtype != G_TYPE_NONE);
 
     if (!object_instance_props_to_g_parameters(context, *object, argc, argv,
                                                gtype,
@@ -1062,7 +1026,7 @@ object_instance_trace(JSTracer *tracer,
     ObjectInstance *priv;
     GList *iter;
 
-    priv = priv_from_js(tracer->context, obj);
+    priv = JS_GetPrivate(tracer->context, obj);
 
     for (iter = priv->signals; iter; iter = iter->next) {
         ConnectData *cd = iter->data;
@@ -1084,8 +1048,7 @@ object_instance_finalize(JSContext *context,
                         g_type_name_from_instance( (GTypeInstance*) priv->gobj) :
                         "<no gobject>",
                         priv ? priv->gobj : NULL);
-    if (priv == NULL)
-        return; /* we are the prototype, not a real instance, so constructor never called */
+    g_assert (priv != NULL);
 
     TRACE(GJS_OBJECT_PROXY_FINALIZE(priv, priv->gobj,
                                     priv->info ? g_base_info_get_namespace((GIBaseInfo*) priv->info) : "_gjs_private",
@@ -1165,7 +1128,7 @@ signal_connection_invalidated (gpointer  user_data,
 
 static JSBool
 real_connect_func(JSContext *context,
-                  uintN      argc,
+                  unsigned   argc,
                   jsval     *vp,
                   gboolean  after)
 {
@@ -1211,8 +1174,7 @@ real_connect_func(JSContext *context,
         return JS_FALSE;
     }
 
-    signal_name = gjs_string_get_ascii(context, argv[0]);
-    if (signal_name == NULL) {
+    if (!gjs_string_to_utf8(context, argv[0], &signal_name)) {
         return JS_FALSE;
     }
 
@@ -1220,7 +1182,7 @@ real_connect_func(JSContext *context,
                              G_OBJECT_TYPE(priv->gobj),
                              &signal_id,
                              &signal_detail,
-                             FALSE)) {
+                             TRUE)) {
         gjs_throw(context, "No signal '%s' on object '%s'",
                      signal_name,
                      g_type_name(G_OBJECT_TYPE(priv->gobj)));
@@ -1239,10 +1201,11 @@ real_connect_func(JSContext *context,
     connect_data->closure = closure;
     g_closure_add_invalidate_notifier(closure, connect_data, signal_connection_invalidated);
 
-    id = g_signal_connect_closure(priv->gobj,
-                                  signal_name,
-                                  closure,
-                                  after);
+    id = g_signal_connect_closure_by_id(priv->gobj,
+                                        signal_id,
+                                        signal_detail,
+                                        closure,
+                                        after);
 
     if (!JS_NewNumberValue(context, id, &retval)) {
         g_signal_handler_disconnect(priv->gobj, id);
@@ -1259,7 +1222,7 @@ real_connect_func(JSContext *context,
 
 static JSBool
 connect_after_func(JSContext *context,
-                   uintN      argc,
+                   unsigned   argc,
                    jsval     *vp)
 {
     return real_connect_func(context, argc, vp, TRUE);
@@ -1267,7 +1230,7 @@ connect_after_func(JSContext *context,
 
 static JSBool
 connect_func(JSContext *context,
-             uintN      argc,
+             unsigned   argc,
              jsval     *vp)
 {
     return real_connect_func(context, argc, vp, FALSE);
@@ -1275,7 +1238,7 @@ connect_func(JSContext *context,
 
 static JSBool
 disconnect_func(JSContext *context,
-                uintN      argc,
+                unsigned   argc,
                 jsval     *vp)
 {
     jsval *argv = JS_ARGV(context, vp);
@@ -1319,7 +1282,7 @@ disconnect_func(JSContext *context,
 
 static JSBool
 emit_func(JSContext *context,
-          uintN      argc,
+          unsigned   argc,
           jsval     *vp)
 {
     jsval *argv = JS_ARGV(context, vp);
@@ -1361,9 +1324,7 @@ emit_func(JSContext *context,
         return JS_FALSE;
     }
 
-    signal_name = gjs_string_get_ascii(context,
-                                                  argv[0]);
-    if (signal_name == NULL)
+    if (!gjs_string_to_utf8(context, argv[0], &signal_name))
         return JS_FALSE;
 
     if (!g_signal_parse_name(signal_name,
@@ -1444,7 +1405,7 @@ emit_func(JSContext *context,
 
 static JSBool
 to_string_func(JSContext *context,
-               uintN      argc,
+               unsigned   argc,
                jsval     *vp)
 {
     JSObject *obj = JS_THIS_OBJECT(context, vp);
@@ -1476,7 +1437,6 @@ static struct JSClass gjs_object_instance_class = {
     "GObject_Object",
     JSCLASS_HAS_PRIVATE |
     JSCLASS_NEW_RESOLVE |
-    JSCLASS_NEW_RESOLVE_GETS_START |
     JSCLASS_MARK_IS_TRACE,
     JS_PropertyStub,
     JS_PropertyStub,
@@ -1498,13 +1458,16 @@ static struct JSClass gjs_object_instance_class = {
 
 static JSBool
 init_func (JSContext *context,
-           uintN      argc,
+           unsigned   argc,
            jsval     *vp)
 {
     jsval *argv = JS_ARGV(context, vp);
     JSObject *obj = JS_THIS_OBJECT(context, vp);
     JSBool ret;
 
+    if (!do_base_typecheck(context, obj, TRUE))
+        return FALSE;
+
     ret = object_instance_init(context, &obj, argc, argv);
 
     if (ret)
@@ -1749,55 +1712,11 @@ gjs_define_object_class(JSContext     *context,
     return TRUE;
 }
 
-/* multiple JSRuntime could have a proxy to the same GObject, in theory
- */
-#define OBJ_KEY_PREFIX_LEN 3
-#define OBJ_KEY_LEN (OBJ_KEY_PREFIX_LEN+sizeof(void*)*2)
-static void
-get_obj_key(JSRuntime *runtime,
-            char      *buf)
-{
-    /* not thread safe, but that's fine for now - just nuke the
-     * cache thingy if we ever need thread safety
-     */
-    static char cached_buf[OBJ_KEY_LEN+1];
-    static JSRuntime *cached_for = NULL;
-
-    if (cached_for != runtime) {
-        unsigned int i;
-        union {
-            const unsigned char bytes[sizeof(void*)];
-            void *ptr;
-        } d;
-        g_assert(sizeof(d) == sizeof(void*));
-
-        buf[0] = 'j';
-        buf[1] = 's';
-        buf[2] = '-';
-        d.ptr = runtime;
-        for (i = 0; i < sizeof(void*); i++) {
-                int offset = OBJ_KEY_PREFIX_LEN+(i*2);
-                buf[offset] = 'a' + ((d.bytes[i] & 0xf0) >> 4);
-                buf[offset+1] = 'a' + (d.bytes[i] & 0x0f);
-        }
-        buf[OBJ_KEY_LEN] = '\0';
-        strcpy(cached_buf, buf);
-        cached_for = runtime;
-        g_assert(strlen(buf) == OBJ_KEY_LEN);
-    } else {
-        strcpy(buf, cached_buf);
-    }
-}
-
 static JSObject*
 peek_js_obj(JSContext *context,
             GObject   *gobj)
 {
-    char buf[OBJ_KEY_LEN+1];
-
-    get_obj_key(JS_GetRuntime(context), buf);
-
-    return g_object_get_data(gobj, buf);
+    return g_object_get_qdata(gobj, gjs_object_priv_quark());
 }
 
 static void
@@ -1805,11 +1724,7 @@ set_js_obj(JSContext *context,
            GObject   *gobj,
            JSObject  *obj)
 {
-    char buf[OBJ_KEY_LEN+1];
-
-    get_obj_key(JS_GetRuntime(context), buf);
-
-    g_object_set_data(gobj, buf, obj);
+    g_object_set_qdata(gobj, gjs_object_priv_quark(), obj);
 }
 
 JSObject*
@@ -2012,7 +1927,7 @@ find_vfunc_info (JSContext *context,
 
 static JSBool
 gjs_hook_up_vfunc(JSContext *cx,
-                  uintN      argc,
+                  unsigned   argc,
                   jsval     *vp)
 {
     jsval *argv = JS_ARGV(cx, vp);
@@ -2231,7 +2146,7 @@ gjs_add_interface(GType instance_type,
 
 static JSBool
 gjs_register_type(JSContext *cx,
-                  uintN      argc,
+                  unsigned   argc,
                   jsval     *vp)
 {
     jsval *argv = JS_ARGV(cx, vp);
@@ -2255,7 +2170,7 @@ gjs_register_type(JSContext *cx,
 	0,    /* n_preallocs */
 	(GInstanceInitFunc) NULL,
     };
-    jsuint i, n_interfaces;
+    guint32 i, n_interfaces;
     GType *iface_types;
 
     JS_BeginRequest(cx);
@@ -2309,8 +2224,8 @@ gjs_register_type(JSContext *cx,
 
     parent_priv = priv_from_js(cx, parent);
 
-    if (!parent_priv)
-        return JS_FALSE;
+    /* We checked parent above, in do_base_typecheck() */
+    g_assert(parent_priv != NULL);
 
     parent_type = parent_priv->gtype;
 
@@ -2350,7 +2265,7 @@ gjs_register_type(JSContext *cx,
 
 static JSBool
 gjs_register_property(JSContext *cx,
-                      uintN      argc,
+                      unsigned   argc,
                       jsval     *vp)
 {
     jsval *argv = JS_ARGV(cx, vp);
@@ -2387,7 +2302,7 @@ gjs_register_property(JSContext *cx,
 
 static JSBool
 gjs_signal_new(JSContext *cx,
-               uintN      argc,
+               unsigned   argc,
                jsval     *vp)
 {
     jsval *argv = JS_ARGV(cx, vp);
