@@ -72,7 +72,6 @@ struct _GjsContext {
 
     guint idle_emit_gc_id;
 
-    guint we_own_runtime : 1;
     guint gc_notifications_enabled : 1;
 };
 
@@ -93,18 +92,17 @@ enum {
     PROP_0,
     PROP_JS_VERSION,
     PROP_SEARCH_PATH,
-    PROP_RUNTIME,
     PROP_GC_NOTIFICATIONS
 };
 
 
-static GStaticMutex contexts_lock = G_STATIC_MUTEX_INIT;
+static GMutex contexts_lock;
 static GList *all_contexts = NULL;
 
 
 static JSBool
 gjs_log(JSContext *context,
-        uintN      argc,
+        unsigned   argc,
         jsval     *vp)
 {
     jsval *argv = JS_ARGV(context, vp);
@@ -148,7 +146,7 @@ gjs_log(JSContext *context,
 
 static JSBool
 gjs_log_error(JSContext *context,
-              uintN      argc,
+              unsigned   argc,
               jsval     *vp)
 {
     jsval *argv = JS_ARGV(context, vp);
@@ -197,7 +195,7 @@ gjs_log_error(JSContext *context,
 
 static JSBool
 gjs_print_parse_args(JSContext *context,
-                     uintN      argc,
+                     unsigned   argc,
                      jsval     *argv,
                      char     **buffer)
 {
@@ -254,7 +252,7 @@ gjs_print_parse_args(JSContext *context,
 
 static JSBool
 gjs_print(JSContext *context,
-          uintN      argc,
+          unsigned   argc,
           jsval     *vp)
 {
     jsval *argv = JS_ARGV(context, vp);
@@ -273,7 +271,7 @@ gjs_print(JSContext *context,
 
 static JSBool
 gjs_printerr(JSContext *context,
-             uintN      argc,
+             unsigned   argc,
              jsval     *vp)
 {
     jsval *argv = JS_ARGV(context, vp);
@@ -319,15 +317,6 @@ gjs_context_class_init(GjsContextClass *klass)
                                     PROP_SEARCH_PATH,
                                     pspec);
 
-    pspec = g_param_spec_pointer("runtime",
-                                 "JSRuntime",
-                                 "A runtime to use instead of creating our own",
-                                 G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
-
-    g_object_class_install_property(object_class,
-                                    PROP_RUNTIME,
-                                    pspec);
-
     pspec = g_param_spec_string("js-version",
                                  "JS Version",
                                  "A string giving the default for the (SpiderMonkey) JavaScript version",
@@ -379,20 +368,13 @@ gjs_context_dispose(GObject *object)
         gjs_debug(GJS_DEBUG_CONTEXT,
                   "Destroying JS context");
 
-        if (js_context->we_own_runtime)
-            gjs_runtime_set_default_context(js_context->runtime, NULL);
         JS_DestroyContext(js_context->context);
         js_context->context = NULL;
     }
 
     if (js_context->runtime != NULL) {
-        if (js_context->we_own_runtime) {
-            /* Cleans up data as well as destroying the runtime. Foreign
-             * contexts aren't supported at the moment; if we supported
-             * them, then the data simply would't get cleaned up.
-             */
-            gjs_runtime_destroy(js_context->runtime);
-        }
+        /* Cleans up data as well as destroying the runtime. */
+        JS_DestroyRuntime(js_context->runtime);
         js_context->runtime = NULL;
     }
 
@@ -418,9 +400,9 @@ gjs_context_finalize(GObject *object)
 
     g_free(js_context->jsversion_string);
 
-    g_static_mutex_lock(&contexts_lock);
+    g_mutex_lock(&contexts_lock);
     all_contexts = g_list_remove(all_contexts, object);
-    g_static_mutex_unlock(&contexts_lock);
+    g_mutex_unlock(&contexts_lock);
 
     G_OBJECT_CLASS(gjs_context_parent_class)->finalize(object);
 }
@@ -541,6 +523,16 @@ static JSLocaleCallbacks gjs_locale_callbacks =
     gjs_locale_to_unicode
 };
 
+static void
+set_c_strings_are_utf8_if_needed (void)
+{
+    static gsize init = 0;
+    if (g_once_init_enter (&init)) {
+        JS_SetCStringsAreUTF8();
+        g_once_init_leave (&init, 1);
+    }
+}
+
 static GObject*
 gjs_context_constructor (GType                  type,
                          guint                  n_construct_properties,
@@ -555,26 +547,23 @@ gjs_context_constructor (GType                  type,
                                                                          n_construct_properties,
                                                                          construct_params);
 
+    set_c_strings_are_utf8_if_needed ();
+
     js_context = GJS_CONTEXT(object);
 
-    if (js_context->runtime == NULL) {
-        js_context->runtime = JS_NewRuntime(32*1024*1024 /* max bytes */);
-        if (js_context->runtime == NULL)
-            gjs_fatal("Failed to create javascript runtime");
-        JS_SetGCParameter(js_context->runtime, JSGC_MAX_BYTES, 0xffffffff);
-        js_context->we_own_runtime = TRUE;
-        gjs_runtime_init(js_context->runtime);
-    }
+    js_context->runtime = JS_NewRuntime(32*1024*1024 /* max bytes */);
+    if (js_context->runtime == NULL)
+        gjs_fatal("Failed to create javascript runtime");
+    JS_SetGCParameter(js_context->runtime, JSGC_MAX_BYTES, 0xffffffff);
 
     js_context->context = JS_NewContext(js_context->runtime, 8192 /* stack chunk size */);
     if (js_context->context == NULL)
         gjs_fatal("Failed to create javascript context");
 
+    JS_SetRuntimePrivate(js_context->runtime, js_context->context);
+
     JS_BeginRequest(js_context->context);
 
-    /* same as firefox, see discussion at
-     * https://bugzilla.mozilla.org/show_bug.cgi?id=420869 */
-    JS_SetScriptStackQuota(js_context->context, 100*1024*1024);
 
     /* JSOPTION_DONT_REPORT_UNCAUGHT: Don't send exceptions to our
      * error report handler; instead leave them set.  This allows us
@@ -652,18 +641,6 @@ gjs_context_constructor (GType                  type,
                            4, GJS_MODULE_PROP_FLAGS))
         gjs_fatal("Failed to define printerr function");
 
-    /* We need to know what the default context is, since it's the context whose
-     * global object is used to load imported JS modules. We currently say that
-     * it's the context of the runtime's owner, but if we needed to support
-     * externally created runtimes, we could define it in some other fashion.
-     */
-    if (js_context->we_own_runtime) {
-        gjs_runtime_set_default_context(js_context->runtime, js_context->context);
-    } else {
-        if (gjs_runtime_get_default_context(js_context->runtime) == NULL)
-            gjs_fatal("GjsContext created for a runtime not owned by GJS");
-    }
-
     /* We create the global-to-runtime root importer with the
      * passed-in search path. If someone else already created
      * the root importer, this is a no-op.
@@ -683,9 +660,7 @@ gjs_context_constructor (GType                  type,
                                   "imports"))
         gjs_fatal("Failed to point 'imports' property at root importer");
 
-    if (js_context->we_own_runtime) {
-        js_context->profiler = gjs_profiler_new(js_context->runtime);
-    }
+    js_context->profiler = gjs_profiler_new(js_context->runtime);
 
     if (!gjs_is_registered_native_module(js_context->context, NULL, "gi"))
         gjs_register_native_module("gi", gjs_define_gi_stuff, GJS_NATIVE_SUPPLIES_MODULE_OBJ);
@@ -702,9 +677,9 @@ gjs_context_constructor (GType                  type,
 
     JS_EndRequest(js_context->context);
 
-    g_static_mutex_lock (&contexts_lock);
+    g_mutex_lock (&contexts_lock);
     all_contexts = g_list_prepend(all_contexts, object);
-    g_static_mutex_unlock (&contexts_lock);
+    g_mutex_unlock (&contexts_lock);
 
     return object;
 }
@@ -746,9 +721,6 @@ gjs_context_set_property (GObject      *object,
     case PROP_SEARCH_PATH:
         js_context->search_path = g_value_dup_boxed(value);
         break;
-    case PROP_RUNTIME:
-        js_context->runtime = g_value_get_pointer(value);
-        break;
     case PROP_JS_VERSION:
         g_free(js_context->jsversion_string);
         if (g_value_get_string (value) == NULL)
@@ -949,11 +921,11 @@ gjs_context_get_all(void)
 {
   GList *result;
   GList *iter;
-  g_static_mutex_lock (&contexts_lock);
+  g_mutex_lock (&contexts_lock);
   result = g_list_copy(all_contexts);
   for (iter = result; iter; iter = iter->next)
     g_object_ref((GObject*)iter->data);
-  g_static_mutex_unlock (&contexts_lock);
+  g_mutex_unlock (&contexts_lock);
   return result;
 }
 
@@ -981,8 +953,6 @@ gjs_context_eval(GjsContext *js_context,
     int line_number;
     jsval retval;
     gboolean success;
-    gunichar2 *u16_script;
-    glong u16_script_len;
 
     g_object_ref(G_OBJECT(js_context));
 
@@ -1008,13 +978,8 @@ gjs_context_eval(GjsContext *js_context,
         }
     }
 
-    if ((u16_script = g_utf8_to_utf16 (script, script_len, NULL, &u16_script_len, error)) == NULL)
-        return FALSE;
-    g_assert (u16_script_len < G_MAXUINT);
-
     /* log and clear exception if it's set (should not be, normally...) */
-    if (gjs_log_exception(js_context->context,
-                             NULL)) {
+    if (gjs_log_exception(js_context->context, NULL)) {
         gjs_debug(GJS_DEBUG_CONTEXT,
                   "Exception was set prior to JS_EvaluateScript()");
     }
@@ -1022,17 +987,18 @@ gjs_context_eval(GjsContext *js_context,
     /* JS_EvaluateScript requires a request even though it sort of seems like
      * it means we're always in a request?
      */
-    gjs_runtime_push_context(js_context->runtime, js_context->context);
     JS_BeginRequest(js_context->context);
 
     retval = JSVAL_VOID;
-    if (!JS_EvaluateUCScript(js_context->context,
-                             js_context->global,
-                             (const jschar*)u16_script,
-                             (guint) u16_script_len,
-                             filename,
-                             line_number,
-                             &retval)) {
+    if (script_len < 0)
+        script_len = strlen(script);
+    if (!JS_EvaluateScript(js_context->context,
+                           js_context->global,
+                           script,
+                           script_len,
+                           filename,
+                           line_number,
+                           &retval)) {
         char *message;
 
         gjs_debug(GJS_DEBUG_CONTEXT,
@@ -1059,7 +1025,6 @@ gjs_context_eval(GjsContext *js_context,
 
         success = FALSE;
     }
-    g_free (u16_script);
 
     gjs_debug(GJS_DEBUG_CONTEXT,
               "Script evaluation succeeded");
@@ -1089,7 +1054,6 @@ gjs_context_eval(GjsContext *js_context,
     }
 
     JS_EndRequest(js_context->context);
-    gjs_runtime_pop_context(js_context->runtime);
 
     g_object_unref(G_OBJECT(js_context));
 
@@ -1151,35 +1115,3 @@ gjs_context_define_string_array(GjsContext  *js_context,
 
     return TRUE;
 }
-
-#if GJS_BUILD_TESTS
-
-void
-gjstest_test_func_gjs_context_construct_destroy(void)
-{
-    GjsContext *context;
-
-    /* Construct twice just to possibly a case where global state from
-     * the first leaks.
-     */
-    context = gjs_context_new ();
-    g_object_unref (context);
-
-    context = gjs_context_new ();
-    g_object_unref (context);
-}
-
-void
-gjstest_test_func_gjs_context_construct_eval(void)
-{
-    GjsContext *context;
-    int estatus;
-    GError *error = NULL;
-
-    context = gjs_context_new ();
-    if (!gjs_context_eval (context, "1+1", -1, "<input>", &estatus, &error))
-        g_error ("%s", error->message);
-    g_object_unref (context);
-}
-
-#endif /* GJS_BUILD_TESTS */
