@@ -47,26 +47,26 @@ struct _GjsProfiler {
     GHashTable *by_file;    /* GjsProfileFunctionKey -> GjsProfileFunction */
 
     GjsProfileData *last_function_entered; /* weak ref to by_file */
-    int64           last_function_exit_time;
+    int64_t         last_function_exit_time;
 };
 
 struct _GjsProfileData {
     /* runtime state tracking */
     GjsProfileData *caller;
-    int64 enter_time;
-    int64 runtime_so_far;
-    uintN recurse_depth;
+    int64_t enter_time;
+    int64_t runtime_so_far;
+    unsigned recurse_depth;
 
     /* final statistics */
-    uintN call_count;
+    unsigned call_count;
 
-    int64 total_time;
-    int64 self_time;
+    int64_t total_time;
+    int64_t self_time;
 };
 
 typedef struct {
     char       *filename;
-    uintN       lineno;
+    unsigned    lineno;
     char       *function_name;
 } GjsProfileFunctionKey;
 
@@ -154,9 +154,8 @@ gjs_profile_function_key_from_js(JSContext             *cx,
      * figure out the name of the called function.
      */
     function_name = JS_GetFunctionId(function);
-    if (function_name)
-        key->function_name = gjs_string_get_ascii(cx, STRING_TO_JSVAL(function_name));
-    else
+    if (!function_name ||
+        !gjs_string_to_utf8(cx, STRING_TO_JSVAL(function_name), &key->function_name))
         key->function_name = g_strdup("(unknown)");
 
     g_assert(key->filename != NULL);
@@ -204,7 +203,7 @@ gjs_profiler_log_call(GjsProfiler  *self,
 {
     GjsProfileFunction *function;
     GjsProfileData *p;
-    int64 now;
+    int64_t now;
 
     function = gjs_profiler_lookup_function(self, cx, fp, before);
     if (!function)
@@ -219,7 +218,7 @@ gjs_profiler_log_call(GjsProfiler  *self,
 
             /* we just exited the caller so account for the time spent */
             if (p->caller) {
-                int64 delta;
+                int64_t delta;
 
                 if (self->last_function_exit_time != 0) {
                     delta = now - self->last_function_exit_time;
@@ -242,7 +241,7 @@ gjs_profiler_log_call(GjsProfiler  *self,
 
         p->recurse_depth += 1;
     } else {
-        int64 delta;
+        int64_t delta;
 
         g_assert(p->recurse_depth > 0);
 
@@ -277,7 +276,7 @@ gjs_profiler_log_call(GjsProfiler  *self,
 static void
 gjs_profiler_new_script_hook(JSContext  *cx,
                              const char *filename,
-                             uintN       lineno,
+                             unsigned    lineno,
                              JSScript   *script,
                              JSFunction *fun,
                              void       *callerdata)
