@@ -117,7 +117,7 @@ format_frame(JSContext* cx, JSStackFrame* fp,
         if (fun) {
 	    JSString* funname = JS_GetFunctionId(fun);
             if (funname)
-                funname_str = gjs_string_get_ascii(cx, STRING_TO_JSVAL(funname));
+              gjs_string_to_utf8(cx, STRING_TO_JSVAL(funname), &funname_str);
 	}
 
         call_obj = JS_GetFrameCallObject(cx, fp);
@@ -245,20 +245,3 @@ gjs_dumpstack(void)
   }
   g_list_free(contexts);
 }
-
-#if GJS_BUILD_TESTS
-void
-gjstest_test_func_gjs_stack_dump(void)
-{
-  GjsContext *context;
-
-  /* TODO this test could be better - maybe expose dumpstack as a JS API
-   * so that we have a JS stack to dump?  At least here we're getting some
-   * coverage.
-   */
-  context = gjs_context_new();
-  gjs_dumpstack();
-  g_object_unref(context);
-  gjs_dumpstack();
-}
-#endif /* GJS_BUILD_TESTS */
