@@ -19,6 +19,8 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 // IN THE SOFTWARE.
 
+const ByteArray = imports.byteArray;
+
 let GLib;
 let originalVariantClass;
 
@@ -66,6 +68,12 @@ function _read_single_type(signature, forceSimple) {
     return [char];
 }
 
+function _makeBytes(byteArray) {
+    if (byteArray instanceof ByteArray.ByteArray)
+        return byteArray.toGBytes();
+    else
+        return new GLib.Bytes(byteArray);
+}
 
 function _pack_variant(signature, value) {
     if (signature.length == 0)
@@ -105,7 +113,7 @@ function _pack_variant(signature, value) {
 	if (value != null)
 	    return GLib.Variant.new_maybe(null, _pack_variant(signature, value))
 	else
-	    return GLib.Variant.new_maybe(GLib.VariantType.new(_read_single_type(signature, false).join('')), null);
+	    return GLib.Variant.new_maybe(new GLib.VariantType(_read_single_type(signature, false).join('')), null);
     case 'a':
 	let arrayType = _read_single_type(signature, false);
 	if (arrayType[0] == 's') {
@@ -114,11 +122,8 @@ function _pack_variant(signature, value) {
 	}
 	if (arrayType[0] == 'y') {
 	    // special case for array of bytes
-	    return GLib.Variant.new_bytestring(value);
-	}
-	if (arrayType[0] == 'a' && arrayType[1] == 'y') {
-	    // special case for array of array of bytes
-	    return GLib.Variant.new_bytestring_array(value);
+	    return GLib.Variant.new_from_bytes(new GLib.VariantType('ay'),
+                                               _makeBytes(value), true);
 	}
 
 	let arrayValue = [];
@@ -136,7 +141,8 @@ function _pack_variant(signature, value) {
 		arrayValue.push(child);
 	    }
 	}
-	return GLib.Variant.new_array(GLib.VariantType.new(arrayType.join('')), arrayValue);
+	return GLib.Variant.new_array(new GLib.VariantType(arrayType.join('')), arrayValue);
+
     case '(':
 	let children = [ ];
 	for (let i = 0; i < value.length; i++) {
@@ -200,7 +206,7 @@ function _unpack_variant(variant, deep) {
 	else
 	    return val;
     case 'a':
-	if (variant.is_of_type(GLib.VariantType.new('a{?*}'))) {
+	if (variant.is_of_type(new GLib.VariantType('a{?*}'))) {
 	    // special case containers
 	    let ret = { };
 	    let nElements = variant.n_children();
@@ -217,6 +223,11 @@ function _unpack_variant(variant, deep) {
 	    }
 	    return ret;
 	}
+        if (variant.is_of_type(new GLib.VariantType('ay'))) {
+            // special case byte arrays
+            return variant.get_data_as_bytes().toArray();
+        }
+
 	// fall through
     case '(':
     case '{':
@@ -245,7 +256,7 @@ function _init() {
     // without checking instanceof
     Error.prototype.matches = function() { return false; }
 
-    this.Variant.new = function (sig, value) {
+    this.Variant._new_internal = function(sig, value) {
 	let signature = Array.prototype.slice.call(sig);
 
 	let variant = _pack_variant(signature, value);
@@ -254,6 +265,11 @@ function _init() {
 
 	return variant;
     }
+
+    // Deprecate version of new GLib.Variant()
+    this.Variant.new = function(sig, value) {
+	return new GLib.Variant(sig, value);
+    }
     this.Variant.prototype.unpack = function() {
 	return _unpack_variant(this, false);
     }
@@ -263,4 +279,8 @@ function _init() {
     this.Variant.prototype.toString = function() {
 	return '[object variant of type "' + this.get_type_string() + '"]';
     }
+
+    this.Bytes.prototype.toArray = function() {
+	return imports.byteArray.fromGBytes(this);
+    }
 }
