@@ -33,7 +33,7 @@
 
 static JSBool
 gjs_address_of(JSContext *context,
-               uintN      argc,
+               unsigned   argc,
                jsval     *vp)
 {
     jsval *argv = JS_ARGV(cx, vp);
@@ -56,7 +56,7 @@ gjs_address_of(JSContext *context,
 
 static JSBool
 gjs_refcount(JSContext *context,
-             uintN      argc,
+             unsigned   argc,
              jsval     *vp)
 {
     jsval *argv = JS_ARGV(cx, vp);
@@ -82,7 +82,7 @@ gjs_refcount(JSContext *context,
 
 static JSBool
 gjs_breakpoint(JSContext *context,
-               uintN      argc,
+               unsigned   argc,
                jsval     *vp)
 {
     jsval *argv = JS_ARGV(cx, vp);
@@ -94,7 +94,7 @@ gjs_breakpoint(JSContext *context,
 
 static JSBool
 gjs_gc(JSContext *context,
-       uintN      argc,
+       unsigned   argc,
        jsval     *vp)
 {
     jsval *argv = JS_ARGV(cx, vp);
@@ -104,48 +104,6 @@ gjs_gc(JSContext *context,
     return JS_TRUE;
 }
 
-static JSBool
-gjs_getpid(JSContext *context,
-           uintN      argc,
-           jsval     *vp)
-{
-    jsval *argv = JS_ARGV(cx, vp);
-    jsval rval;
-    if (!gjs_parse_args(context, "getpid", "", argc, argv))
-        return JS_FALSE;
-    rval = INT_TO_JSVAL(getpid());
-    JS_SET_RVAL(context, vp, rval);
-    return JS_TRUE;
-}
-
-static JSBool
-gjs_getuid(JSContext *context,
-           uintN      argc,
-           jsval     *vp)
-{
-    jsval *argv = JS_ARGV(cx, vp);
-    jsval rval;
-    if (!gjs_parse_args(context, "getuid", "", argc, argv))
-        return JS_FALSE;
-    rval = INT_TO_JSVAL(getuid());
-    JS_SET_RVAL(context, vp, rval);
-    return JS_TRUE;
-}
-
-static JSBool
-gjs_getgid(JSContext *context,
-           uintN      argc,
-           jsval     *vp)
-{
-    jsval *argv = JS_ARGV(cx, vp);
-    jsval rval;
-    if (!gjs_parse_args(context, "getgid", "", argc, argv))
-        return JS_FALSE;
-    rval = INT_TO_JSVAL(getgid());
-    JS_SET_RVAL(context, vp, rval);
-    return JS_TRUE;
-}
-
 JSBool
 gjs_js_define_system_stuff(JSContext *context,
                            JSObject  *module)
@@ -174,24 +132,6 @@ gjs_js_define_system_stuff(JSContext *context,
                            0, GJS_MODULE_PROP_FLAGS))
         return JS_FALSE;
 
-    if (!JS_DefineFunction(context, module,
-                           "getpid",
-                           (JSNative) gjs_getpid,
-                           0, GJS_MODULE_PROP_FLAGS))
-        return FALSE;
-
-    if (!JS_DefineFunction(context, module,
-                           "getuid",
-                           (JSNative) gjs_getuid,
-                           0, GJS_MODULE_PROP_FLAGS))
-        return FALSE;
-
-    if (!JS_DefineFunction(context, module,
-                           "getgid",
-                           (JSNative) gjs_getgid,
-                           0, GJS_MODULE_PROP_FLAGS))
-        return FALSE;
-
     return JS_TRUE;
 }
 
