@@ -1,6 +1,7 @@
 // application/javascript;version=1.8
 // This used to be called "Everything"
 const Everything = imports.gi.Regress;
+const WarnLib = imports.gi.WarnLib;
 if (!('assertEquals' in this)) { /* allow running this test standalone */
     imports.lang.copyPublicProperties(imports.jsUnit, this);
     gjstestRun = function() { return imports.jsUnit.gjstestRun(window); };
@@ -548,6 +549,62 @@ function testGError() {
 	assertEquals(Gio.IOErrorEnum.PERMISSION_DENIED, e.code);
 	assertEquals('regression test owned error', e.message);
     });
+
+    // Calling matches() on an unpaired error used to assert:
+    // https://bugzilla.gnome.org/show_bug.cgi?id=689482
+    try {
+	WarnLib.throw_unpaired();
+	assertTrue(false);
+    } catch (e) {
+	assertFalse(e.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.NOT_FOUND));
+    }
+}
+
+function testWrongClassGObject() {
+    /* Function calls */
+    // Everything.func_obj_null_in expects a Everything.TestObj
+    assertRaises(function() {
+	Everything.func_obj_null_in(new Gio.SimpleAction);
+    });
+    assertRaises(function() {
+	Everything.func_obj_null_in(new GLib.KeyFile);
+    });
+    assertRaises(function() {
+	Everything.func_obj_null_in(Gio.File.new_for_path('/'));
+    });
+    Everything.func_obj_null_in(new Everything.TestSubObj);
+
+    /* Method calls */
+    assertRaises(function() {
+	Everything.TestObj.prototype.instance_method.call(new Gio.SimpleAction);
+    });
+    assertRaises(function() {
+	Everything.TestObj.prototype.instance_method.call(new GLib.KeyFile);
+    });
+    Everything.TestObj.prototype.instance_method.call(new Everything.TestSubObj);
+}
+
+function testWrongClassGBoxed() {
+    let simpleBoxed = new Everything.TestSimpleBoxedA;
+    // simpleBoxed.equals expects a Everything.TestSimpleBoxedA
+    assertRaises(function() {
+	simpleBoxed.equals(new Gio.SimpleAction);
+    })
+    assertRaises(function() {
+	simpleBoxed.equals(new Everything.TestObj);
+    })
+    assertRaises(function() {
+	simpleBoxed.equals(new GLib.KeyFile);
+    })
+    assertTrue(simpleBoxed.equals(simpleBoxed));
+
+    assertRaises(function() {
+	Everything.TestSimpleBoxedA.prototype.copy.call(new Gio.SimpleAction);
+    });
+    assertRaises(function() {
+	Everything.TestSimpleBoxedA.prototype.copy.call(new GLib.KeyFile);
+    })
+    Everything.TestSimpleBoxedA.prototype.copy.call(simpleBoxed);
 }
 
 gjstestRun();
